
/******************************************************************************/
/* termvar.c 							              */
/* This is the linux Communication Test Program for Advantech PCL card
 *
 * This file is part of pclterm
 * copyright (c)2000 ,ADVANTECH Co,Ltd
 *
 * pclterm is free software;you can redistribute it and/or modify it
 * under the terms of version 2 of GNU General Public License as
 * published by the Free Software Foundation.
*/
/*	term directory used global variable define			      */
/******************************************************************************/

#include	"../pclwinlib/win.h"
#include	"term.h"


int	SioFd=-1;
struct dterm_info	DTerm;
struct WItem	BaudRateMenuItem[] = {
	{F_Active, "50"},
	{F_Active, "75"},
	{F_Active, "110"},
	{F_Active, "134"},
	{F_Active, "150"},
	{F_Active, "200"},
	{F_Active, "300"},
	{F_Active, "600"},
	{F_Active, "1200"},
	{F_Active, "1800"},
	{F_Active, "2400"},
	{F_Active, "4800"},
	{F_Active, "9600"},
	{F_Active, "19200"},
	{F_Active, "38400"},
	{F_Active, "57600"},
	{F_Active, "115200"},
	{F_Active, "230400"},
	{F_Active, "460800"},
	{F_Active, "921600"}
};
struct WSelect	BaudRateMenu = {
	-1, 3, 0, 20, 0, 0, 0, 20, BaudRateMenuItem
};
struct WItem	ParityMenuItem[] = {
	{F_Active, " None "},
	{F_Active, "  Odd "},
	{F_Active, " Even "}
};
struct WSelect	ParityMenu = {
	-1, 10, 0, 3, 0, 0, 0, 3, ParityMenuItem
};
struct WItem	DataBitMenuItem[] = {
	{F_Active, " 5 "},
	{F_Active, " 6 "},
	{F_Active, " 7 "},
	{F_Active, " 8 "}
};
struct WSelect	DataBitMenu = {
	-1, 10, 0, 4, 0, 0, 0, 4, DataBitMenuItem
};
struct WItem	StopBitMenuItem[] = {
	{F_Active, " 1 "},
	{F_Active, " 2 "}
};
struct WSelect	StopBitMenu = {
	-1, 10, 0, 2, 0, 0, 0, 2, StopBitMenuItem
};
struct WItem	YesNoMenuItem[] = {
	{F_Active, "No "},
	{F_Active, "Yes"}
};
struct WSelect	YesNoMenu = {
	-1, 10, 0, 2, 0, 0, 0, 2, YesNoMenuItem
};

struct WItem	FlowMenuItem[] = {
	{F_Active, "  None  ", 'N'},
	{F_Active, "RTS/CTS ", 'R'},
	{F_Active, "XON/XOFF", 'X'},
	{F_Active, "  Both  ", 'B'}
};
struct WSelect 	FlowMenu = {
	-1, 10, 0, 4, 0, 0, 0, 4, FlowMenuItem
};


struct WItem	EnterMenuItem[] = {
	{F_Active, "CR-LF"},
	{F_Active, " CR  "},
	{F_Active, " LF  "},
	{F_Active, "LF-CR"}
};
struct WSelect	EnterMenu = {
	-1, 10, 0, 4, 0, 0, 0, 4, EnterMenuItem
};

long BaudMapTab[] = {
     B50, B75, B110, B134, B150, B200, B300, B600, B1200,
     B1800,B2400,B4800,B9600,B19200,B38400,B57600,B115200,
     B230400,B460800,B921600
};
